#!/usr/bin/env python

#
# Generated Sun Jun 14 13:41:29 2015 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase

from saml2.ws import wsaddr as wsa
from saml2.ws import wssec as wsse
from saml2.ws import wsutil as wsu
from saml2.ws import wspol as wsp

NAMESPACE = 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/'

class RequestSecurityTokenType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenType element """

    c_tag = 'RequestSecurityTokenType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Context'] = ('context', 'anyURI', False)

    def __init__(self,
            context=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.context=context

def request_security_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenType_, xml_string)


class TokenType(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:TokenType element """

    c_tag = 'TokenType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def token_type_from_string(xml_string):
    return saml2.create_class_from_xml_string(TokenType, xml_string)


class RequestTypeOpenEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestTypeOpenEnum element """

    c_tag = 'RequestTypeOpenEnum'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def request_type_open_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestTypeOpenEnum_, xml_string)


class RequestTypeEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestTypeEnum element """

    c_tag = 'RequestTypeEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xs:anyURI', 'enumeration': ['http://docs.oasis-open.org/ws-sx/ws-trust/200512/Issue', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/Renew', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/Cancel', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/STSCancel', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/Validate']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def request_type_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestTypeEnum_, xml_string)


class RequestSecurityTokenResponseType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenResponseType element """

    c_tag = 'RequestSecurityTokenResponseType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Context'] = ('context', 'anyURI', False)

    def __init__(self,
            context=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.context=context

def request_security_token_response_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenResponseType_, xml_string)


class RequestedSecurityTokenType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedSecurityTokenType element """

    c_tag = 'RequestedSecurityTokenType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def requested_security_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedSecurityTokenType_, xml_string)


class BinarySecretTypeEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinarySecretTypeEnum element """

    c_tag = 'BinarySecretTypeEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xs:anyURI', 'enumeration': ['http://docs.oasis-open.org/ws-sx/ws-trust/200512/AsymmetricKey', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/SymmetricKey', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/Nonce']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def binary_secret_type_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecretTypeEnum_, xml_string)


class BinarySecretTypeOpenEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinarySecretTypeOpenEnum element """

    c_tag = 'BinarySecretTypeOpenEnum'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def binary_secret_type_open_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecretTypeOpenEnum_, xml_string)


class ClaimsType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ClaimsType element """

    c_tag = 'ClaimsType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Dialect'] = ('dialect', 'anyURI', False)

    def __init__(self,
            dialect=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.dialect=dialect

def claims_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ClaimsType_, xml_string)


class EntropyType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:EntropyType element """

    c_tag = 'EntropyType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def entropy_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EntropyType_, xml_string)


class LifetimeType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:LifetimeType element """

    c_tag = 'LifetimeType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Created'] = ('created', wsu.Created)
    c_cardinality['created'] = {"min":0, "max":1}
    c_children['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Expires'] = ('expires', wsu.Expires)
    c_cardinality['expires'] = {"min":0, "max":1}
    c_child_order.extend(['created', 'expires'])

    def __init__(self,
            created=None,
            expires=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.created=created
        self.expires=expires

def lifetime_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(LifetimeType_, xml_string)


class RequestSecurityTokenCollectionType_RequestSecurityToken(RequestSecurityTokenType_):

    c_tag = 'RequestSecurityToken'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenType_.c_children.copy()
    c_attributes = RequestSecurityTokenType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenType_.c_cardinality.copy()

def request_security_token_collection_type__request_security_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenCollectionType_RequestSecurityToken, xml_string)


class RequestSecurityTokenCollectionType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenCollectionType element """

    c_tag = 'RequestSecurityTokenCollectionType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}RequestSecurityToken'] = ('request_security_token', [RequestSecurityTokenCollectionType_RequestSecurityToken])
    c_cardinality['request_security_token'] = {"min":2}
    c_child_order.extend(['request_security_token'])

    def __init__(self,
            request_security_token=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.request_security_token=request_security_token or []

def request_security_token_collection_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenCollectionType_, xml_string)


class ComputedKeyEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ComputedKeyEnum element """

    c_tag = 'ComputedKeyEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xs:anyURI', 'enumeration': ['http://docs.oasis-open.org/ws-sx/ws-trust/200512/CK/PSHA1', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/CK/HASH']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def computed_key_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(ComputedKeyEnum_, xml_string)


class ComputedKeyOpenEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ComputedKeyOpenEnum element """

    c_tag = 'ComputedKeyOpenEnum'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def computed_key_open_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(ComputedKeyOpenEnum_, xml_string)


class RequestedReferenceType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedReferenceType element """

    c_tag = 'RequestedReferenceType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd}SecurityTokenReference'] = ('security_token_reference', wsse.SecurityTokenReference)
    c_child_order.extend(['security_token_reference'])

    def __init__(self,
            security_token_reference=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.security_token_reference=security_token_reference

def requested_reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedReferenceType_, xml_string)


class RequestedProofTokenType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedProofTokenType element """

    c_tag = 'RequestedProofTokenType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def requested_proof_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedProofTokenType_, xml_string)


class RenewTargetType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RenewTargetType element """

    c_tag = 'RenewTargetType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def renew_target_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RenewTargetType_, xml_string)


class AllowPostdatingType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:AllowPostdatingType element """

    c_tag = 'AllowPostdatingType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def allow_postdating_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AllowPostdatingType_, xml_string)


class RenewingType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RenewingType element """

    c_tag = 'RenewingType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Allow'] = ('allow', 'boolean', False)
    c_attributes['OK'] = ('ok', 'boolean', False)

    def __init__(self,
            allow=None,
            ok=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.allow=allow
        self.ok=ok

def renewing_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RenewingType_, xml_string)


class CancelTargetType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:CancelTargetType element """

    c_tag = 'CancelTargetType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def cancel_target_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(CancelTargetType_, xml_string)


class RequestedTokenCancelledType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedTokenCancelledType element """

    c_tag = 'RequestedTokenCancelledType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def requested_token_cancelled_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedTokenCancelledType_, xml_string)


class ValidateTargetType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ValidateTargetType element """

    c_tag = 'ValidateTargetType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def validate_target_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ValidateTargetType_, xml_string)


class StatusCodeEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:StatusCodeEnum element """

    c_tag = 'StatusCodeEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xs:anyURI', 'enumeration': ['http://docs.oasis-open.org/ws-sx/ws-trust/200512/status/valid', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/status/invalid']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def status_code_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(StatusCodeEnum_, xml_string)


class StatusCodeOpenEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:StatusCodeOpenEnum element """

    c_tag = 'StatusCodeOpenEnum'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def status_code_open_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(StatusCodeOpenEnum_, xml_string)


class Challenge(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Challenge element """

    c_tag = 'Challenge'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def challenge_from_string(xml_string):
    return saml2.create_class_from_xml_string(Challenge, xml_string)


class BinaryExchangeType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinaryExchangeType element """

    c_tag = 'BinaryExchangeType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['ValueType'] = ('value_type', 'anyURI', True)
    c_attributes['EncodingType'] = ('encoding_type', 'anyURI', True)

    def __init__(self,
            value_type=None,
            encoding_type=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.value_type=value_type
        self.encoding_type=encoding_type

def binary_exchange_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinaryExchangeType_, xml_string)


class RequestKETType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestKETType element """

    c_tag = 'RequestKETType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def request_ket_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestKETType_, xml_string)


class KeyExchangeTokenType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyExchangeTokenType element """

    c_tag = 'KeyExchangeTokenType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_exchange_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyExchangeTokenType_, xml_string)


class CombinedHash(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:CombinedHash element """

    c_tag = 'CombinedHash'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'base64Binary'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def combined_hash_from_string(xml_string):
    return saml2.create_class_from_xml_string(CombinedHash, xml_string)


class OnBehalfOfType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:OnBehalfOfType element """

    c_tag = 'OnBehalfOfType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def on_behalf_of_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(OnBehalfOfType_, xml_string)


class Issuer(wsa.EndpointReferenceType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Issuer element """

    c_tag = 'Issuer'
    c_namespace = NAMESPACE
    c_children = wsa.EndpointReferenceType_.c_children.copy()
    c_attributes = wsa.EndpointReferenceType_.c_attributes.copy()
    c_child_order = wsa.EndpointReferenceType_.c_child_order[:]
    c_cardinality = wsa.EndpointReferenceType_.c_cardinality.copy()

def issuer_from_string(xml_string):
    return saml2.create_class_from_xml_string(Issuer, xml_string)


class AuthenticationType(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:AuthenticationType element """

    c_tag = 'AuthenticationType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def authentication_type_from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthenticationType, xml_string)


class KeyTypeEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyTypeEnum element """

    c_tag = 'KeyTypeEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xs:anyURI', 'enumeration': ['http://docs.oasis-open.org/ws-sx/ws-trust/200512/PublicKey', 'http://docs.oasis-open.org/ws-sx/ws-trust/200512/SymmetricKey', 'http://docs.oasis-open.org/wssx/wstrust/200512/Bearer']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_type_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyTypeEnum_, xml_string)


class KeyTypeOpenEnum_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyTypeOpenEnum element """

    c_tag = 'KeyTypeOpenEnum'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_type_open_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyTypeOpenEnum_, xml_string)


class KeySize(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeySize element """

    c_tag = 'KeySize'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'unsignedInt'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_size_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeySize, xml_string)


class SignatureAlgorithm(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:SignatureAlgorithm element """

    c_tag = 'SignatureAlgorithm'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def signature_algorithm_from_string(xml_string):
    return saml2.create_class_from_xml_string(SignatureAlgorithm, xml_string)


class EncryptionAlgorithm(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:EncryptionAlgorithm element """

    c_tag = 'EncryptionAlgorithm'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def encryption_algorithm_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionAlgorithm, xml_string)


class CanonicalizationAlgorithm(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:CanonicalizationAlgorithm element """

    c_tag = 'CanonicalizationAlgorithm'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def canonicalization_algorithm_from_string(xml_string):
    return saml2.create_class_from_xml_string(CanonicalizationAlgorithm, xml_string)


class ComputedKeyAlgorithm(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ComputedKeyAlgorithm element """

    c_tag = 'ComputedKeyAlgorithm'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def computed_key_algorithm_from_string(xml_string):
    return saml2.create_class_from_xml_string(ComputedKeyAlgorithm, xml_string)


class EncryptionType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:EncryptionType element """

    c_tag = 'EncryptionType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def encryption_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptionType_, xml_string)


class ProofEncryptionType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ProofEncryptionType element """

    c_tag = 'ProofEncryptionType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def proof_encryption_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ProofEncryptionType_, xml_string)


class UseKeyType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:UseKeyType element """

    c_tag = 'UseKeyType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Sig'] = ('sig', 'anyURI', False)

    def __init__(self,
            sig=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.sig=sig

def use_key_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(UseKeyType_, xml_string)


class KeyWrapAlgorithm(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyWrapAlgorithm element """

    c_tag = 'KeyWrapAlgorithm'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def key_wrap_algorithm_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyWrapAlgorithm, xml_string)


class SignWith(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:SignWith element """

    c_tag = 'SignWith'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def sign_with_from_string(xml_string):
    return saml2.create_class_from_xml_string(SignWith, xml_string)


class EncryptWith(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:EncryptWith element """

    c_tag = 'EncryptWith'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def encrypt_with_from_string(xml_string):
    return saml2.create_class_from_xml_string(EncryptWith, xml_string)


class DelegateToType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:DelegateToType element """

    c_tag = 'DelegateToType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def delegate_to_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(DelegateToType_, xml_string)


class Forwardable(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Forwardable element """

    c_tag = 'Forwardable'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'boolean'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def forwardable_from_string(xml_string):
    return saml2.create_class_from_xml_string(Forwardable, xml_string)


class Delegatable(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Delegatable element """

    c_tag = 'Delegatable'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'boolean'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def delegatable_from_string(xml_string):
    return saml2.create_class_from_xml_string(Delegatable, xml_string)


class ParticipantType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ParticipantType element """

    c_tag = 'ParticipantType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def participant_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ParticipantType_, xml_string)


class RequestSecurityToken(RequestSecurityTokenType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityToken element """

    c_tag = 'RequestSecurityToken'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenType_.c_children.copy()
    c_attributes = RequestSecurityTokenType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenType_.c_cardinality.copy()

def request_security_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityToken, xml_string)


class RequestType(RequestTypeOpenEnum_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestType element """

    c_tag = 'RequestType'
    c_namespace = NAMESPACE
    c_children = RequestTypeOpenEnum_.c_children.copy()
    c_attributes = RequestTypeOpenEnum_.c_attributes.copy()
    c_child_order = RequestTypeOpenEnum_.c_child_order[:]
    c_cardinality = RequestTypeOpenEnum_.c_cardinality.copy()

def request_type_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestType, xml_string)


class RequestSecurityTokenResponse(RequestSecurityTokenResponseType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenResponse element """

    c_tag = 'RequestSecurityTokenResponse'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenResponseType_.c_children.copy()
    c_attributes = RequestSecurityTokenResponseType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenResponseType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenResponseType_.c_cardinality.copy()

def request_security_token_response_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenResponse, xml_string)


class RequestedSecurityToken(RequestedSecurityTokenType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedSecurityToken element """

    c_tag = 'RequestedSecurityToken'
    c_namespace = NAMESPACE
    c_children = RequestedSecurityTokenType_.c_children.copy()
    c_attributes = RequestedSecurityTokenType_.c_attributes.copy()
    c_child_order = RequestedSecurityTokenType_.c_child_order[:]
    c_cardinality = RequestedSecurityTokenType_.c_cardinality.copy()

def requested_security_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedSecurityToken, xml_string)


class BinarySecretType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinarySecretType element """

    c_tag = 'BinarySecretType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'base64Binary'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['Type'] = ('type', BinarySecretTypeOpenEnum_, False)

    def __init__(self,
            type=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.type=type

def binary_secret_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecretType_, xml_string)


class Claims(ClaimsType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Claims element """

    c_tag = 'Claims'
    c_namespace = NAMESPACE
    c_children = ClaimsType_.c_children.copy()
    c_attributes = ClaimsType_.c_attributes.copy()
    c_child_order = ClaimsType_.c_child_order[:]
    c_cardinality = ClaimsType_.c_cardinality.copy()

def claims_from_string(xml_string):
    return saml2.create_class_from_xml_string(Claims, xml_string)


class Entropy(EntropyType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Entropy element """

    c_tag = 'Entropy'
    c_namespace = NAMESPACE
    c_children = EntropyType_.c_children.copy()
    c_attributes = EntropyType_.c_attributes.copy()
    c_child_order = EntropyType_.c_child_order[:]
    c_cardinality = EntropyType_.c_cardinality.copy()

def entropy_from_string(xml_string):
    return saml2.create_class_from_xml_string(Entropy, xml_string)


class Lifetime(LifetimeType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Lifetime element """

    c_tag = 'Lifetime'
    c_namespace = NAMESPACE
    c_children = LifetimeType_.c_children.copy()
    c_attributes = LifetimeType_.c_attributes.copy()
    c_child_order = LifetimeType_.c_child_order[:]
    c_cardinality = LifetimeType_.c_cardinality.copy()

def lifetime_from_string(xml_string):
    return saml2.create_class_from_xml_string(Lifetime, xml_string)


class RequestSecurityTokenCollection(RequestSecurityTokenCollectionType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenCollection element """

    c_tag = 'RequestSecurityTokenCollection'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenCollectionType_.c_children.copy()
    c_attributes = RequestSecurityTokenCollectionType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenCollectionType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenCollectionType_.c_cardinality.copy()

def request_security_token_collection_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenCollection, xml_string)


class RequestSecurityTokenResponseCollectionType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenResponseCollectionType element """

    c_tag = 'RequestSecurityTokenResponseCollectionType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}RequestSecurityTokenResponse'] = ('request_security_token_response', [RequestSecurityTokenResponse])
    c_cardinality['request_security_token_response'] = {"min":1}
    c_child_order.extend(['request_security_token_response'])

    def __init__(self,
            request_security_token_response=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.request_security_token_response=request_security_token_response or []

def request_security_token_response_collection_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenResponseCollectionType_, xml_string)


class ComputedKey(ComputedKeyOpenEnum_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ComputedKey element """

    c_tag = 'ComputedKey'
    c_namespace = NAMESPACE
    c_children = ComputedKeyOpenEnum_.c_children.copy()
    c_attributes = ComputedKeyOpenEnum_.c_attributes.copy()
    c_child_order = ComputedKeyOpenEnum_.c_child_order[:]
    c_cardinality = ComputedKeyOpenEnum_.c_cardinality.copy()

def computed_key_from_string(xml_string):
    return saml2.create_class_from_xml_string(ComputedKey, xml_string)


class RequestedAttachedReference(RequestedReferenceType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedAttachedReference element """

    c_tag = 'RequestedAttachedReference'
    c_namespace = NAMESPACE
    c_children = RequestedReferenceType_.c_children.copy()
    c_attributes = RequestedReferenceType_.c_attributes.copy()
    c_child_order = RequestedReferenceType_.c_child_order[:]
    c_cardinality = RequestedReferenceType_.c_cardinality.copy()

def requested_attached_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedAttachedReference, xml_string)


class RequestedUnattachedReference(RequestedReferenceType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedUnattachedReference element """

    c_tag = 'RequestedUnattachedReference'
    c_namespace = NAMESPACE
    c_children = RequestedReferenceType_.c_children.copy()
    c_attributes = RequestedReferenceType_.c_attributes.copy()
    c_child_order = RequestedReferenceType_.c_child_order[:]
    c_cardinality = RequestedReferenceType_.c_cardinality.copy()

def requested_unattached_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedUnattachedReference, xml_string)


class RequestedProofToken(RequestedProofTokenType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedProofToken element """

    c_tag = 'RequestedProofToken'
    c_namespace = NAMESPACE
    c_children = RequestedProofTokenType_.c_children.copy()
    c_attributes = RequestedProofTokenType_.c_attributes.copy()
    c_child_order = RequestedProofTokenType_.c_child_order[:]
    c_cardinality = RequestedProofTokenType_.c_cardinality.copy()

def requested_proof_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedProofToken, xml_string)


class IssuedTokens(RequestSecurityTokenResponseCollectionType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:IssuedTokens element """

    c_tag = 'IssuedTokens'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenResponseCollectionType_.c_children.copy()
    c_attributes = RequestSecurityTokenResponseCollectionType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenResponseCollectionType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenResponseCollectionType_.c_cardinality.copy()

def issued_tokens_from_string(xml_string):
    return saml2.create_class_from_xml_string(IssuedTokens, xml_string)


class RenewTarget(RenewTargetType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RenewTarget element """

    c_tag = 'RenewTarget'
    c_namespace = NAMESPACE
    c_children = RenewTargetType_.c_children.copy()
    c_attributes = RenewTargetType_.c_attributes.copy()
    c_child_order = RenewTargetType_.c_child_order[:]
    c_cardinality = RenewTargetType_.c_cardinality.copy()

def renew_target_from_string(xml_string):
    return saml2.create_class_from_xml_string(RenewTarget, xml_string)


class AllowPostdating(AllowPostdatingType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:AllowPostdating element """

    c_tag = 'AllowPostdating'
    c_namespace = NAMESPACE
    c_children = AllowPostdatingType_.c_children.copy()
    c_attributes = AllowPostdatingType_.c_attributes.copy()
    c_child_order = AllowPostdatingType_.c_child_order[:]
    c_cardinality = AllowPostdatingType_.c_cardinality.copy()

def allow_postdating_from_string(xml_string):
    return saml2.create_class_from_xml_string(AllowPostdating, xml_string)


class Renewing(RenewingType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Renewing element """

    c_tag = 'Renewing'
    c_namespace = NAMESPACE
    c_children = RenewingType_.c_children.copy()
    c_attributes = RenewingType_.c_attributes.copy()
    c_child_order = RenewingType_.c_child_order[:]
    c_cardinality = RenewingType_.c_cardinality.copy()

def renewing_from_string(xml_string):
    return saml2.create_class_from_xml_string(Renewing, xml_string)


class CancelTarget(CancelTargetType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:CancelTarget element """

    c_tag = 'CancelTarget'
    c_namespace = NAMESPACE
    c_children = CancelTargetType_.c_children.copy()
    c_attributes = CancelTargetType_.c_attributes.copy()
    c_child_order = CancelTargetType_.c_child_order[:]
    c_cardinality = CancelTargetType_.c_cardinality.copy()

def cancel_target_from_string(xml_string):
    return saml2.create_class_from_xml_string(CancelTarget, xml_string)


class RequestedTokenCancelled(RequestedTokenCancelledType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestedTokenCancelled element """

    c_tag = 'RequestedTokenCancelled'
    c_namespace = NAMESPACE
    c_children = RequestedTokenCancelledType_.c_children.copy()
    c_attributes = RequestedTokenCancelledType_.c_attributes.copy()
    c_child_order = RequestedTokenCancelledType_.c_child_order[:]
    c_cardinality = RequestedTokenCancelledType_.c_cardinality.copy()

def requested_token_cancelled_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestedTokenCancelled, xml_string)


class ValidateTarget(ValidateTargetType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ValidateTarget element """

    c_tag = 'ValidateTarget'
    c_namespace = NAMESPACE
    c_children = ValidateTargetType_.c_children.copy()
    c_attributes = ValidateTargetType_.c_attributes.copy()
    c_child_order = ValidateTargetType_.c_child_order[:]
    c_cardinality = ValidateTargetType_.c_cardinality.copy()

def validate_target_from_string(xml_string):
    return saml2.create_class_from_xml_string(ValidateTarget, xml_string)


class StatusType_Code(StatusCodeOpenEnum_):

    c_tag = 'Code'
    c_namespace = NAMESPACE
    c_children = StatusCodeOpenEnum_.c_children.copy()
    c_attributes = StatusCodeOpenEnum_.c_attributes.copy()
    c_child_order = StatusCodeOpenEnum_.c_child_order[:]
    c_cardinality = StatusCodeOpenEnum_.c_cardinality.copy()

def status_type__code_from_string(xml_string):
    return saml2.create_class_from_xml_string(StatusType_Code, xml_string)


class StatusType_Reason(SamlBase):

    c_tag = 'Reason'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def status_type__reason_from_string(xml_string):
    return saml2.create_class_from_xml_string(StatusType_Reason, xml_string)


class StatusType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:StatusType element """

    c_tag = 'StatusType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}Code'] = ('code', StatusType_Code)
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}Reason'] = ('reason', StatusType_Reason)
    c_cardinality['reason'] = {"min":0, "max":1}
    c_child_order.extend(['code', 'reason'])

    def __init__(self,
            code=None,
            reason=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.code=code
        self.reason=reason

def status_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(StatusType_, xml_string)


class SignChallengeType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:SignChallengeType element """

    c_tag = 'SignChallengeType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}Challenge'] = ('challenge', Challenge)
    c_child_order.extend(['challenge'])

    def __init__(self,
            challenge=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.challenge=challenge

def sign_challenge_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SignChallengeType_, xml_string)


class BinaryExchange(BinaryExchangeType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinaryExchange element """

    c_tag = 'BinaryExchange'
    c_namespace = NAMESPACE
    c_children = BinaryExchangeType_.c_children.copy()
    c_attributes = BinaryExchangeType_.c_attributes.copy()
    c_child_order = BinaryExchangeType_.c_child_order[:]
    c_cardinality = BinaryExchangeType_.c_cardinality.copy()

def binary_exchange_from_string(xml_string):
    return saml2.create_class_from_xml_string(BinaryExchange, xml_string)


class RequestKET(RequestKETType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestKET element """

    c_tag = 'RequestKET'
    c_namespace = NAMESPACE
    c_children = RequestKETType_.c_children.copy()
    c_attributes = RequestKETType_.c_attributes.copy()
    c_child_order = RequestKETType_.c_child_order[:]
    c_cardinality = RequestKETType_.c_cardinality.copy()

def request_ket_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestKET, xml_string)


class KeyExchangeToken(KeyExchangeTokenType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyExchangeToken element """

    c_tag = 'KeyExchangeToken'
    c_namespace = NAMESPACE
    c_children = KeyExchangeTokenType_.c_children.copy()
    c_attributes = KeyExchangeTokenType_.c_attributes.copy()
    c_child_order = KeyExchangeTokenType_.c_child_order[:]
    c_cardinality = KeyExchangeTokenType_.c_cardinality.copy()

def key_exchange_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyExchangeToken, xml_string)


class AuthenticatorType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:AuthenticatorType element """

    c_tag = 'AuthenticatorType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}CombinedHash'] = ('combined_hash', CombinedHash)
    c_cardinality['combined_hash'] = {"min":0, "max":1}
    c_child_order.extend(['combined_hash'])

    def __init__(self,
            combined_hash=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.combined_hash=combined_hash

def authenticator_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(AuthenticatorType_, xml_string)


class OnBehalfOf(OnBehalfOfType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:OnBehalfOf element """

    c_tag = 'OnBehalfOf'
    c_namespace = NAMESPACE
    c_children = OnBehalfOfType_.c_children.copy()
    c_attributes = OnBehalfOfType_.c_attributes.copy()
    c_child_order = OnBehalfOfType_.c_child_order[:]
    c_cardinality = OnBehalfOfType_.c_cardinality.copy()

def on_behalf_of_from_string(xml_string):
    return saml2.create_class_from_xml_string(OnBehalfOf, xml_string)


class KeyType(KeyTypeOpenEnum_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:KeyType element """

    c_tag = 'KeyType'
    c_namespace = NAMESPACE
    c_children = KeyTypeOpenEnum_.c_children.copy()
    c_attributes = KeyTypeOpenEnum_.c_attributes.copy()
    c_child_order = KeyTypeOpenEnum_.c_child_order[:]
    c_cardinality = KeyTypeOpenEnum_.c_cardinality.copy()

def key_type_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyType, xml_string)


class Encryption(EncryptionType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Encryption element """

    c_tag = 'Encryption'
    c_namespace = NAMESPACE
    c_children = EncryptionType_.c_children.copy()
    c_attributes = EncryptionType_.c_attributes.copy()
    c_child_order = EncryptionType_.c_child_order[:]
    c_cardinality = EncryptionType_.c_cardinality.copy()

def encryption_from_string(xml_string):
    return saml2.create_class_from_xml_string(Encryption, xml_string)


class ProofEncryption(ProofEncryptionType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ProofEncryption element """

    c_tag = 'ProofEncryption'
    c_namespace = NAMESPACE
    c_children = ProofEncryptionType_.c_children.copy()
    c_attributes = ProofEncryptionType_.c_attributes.copy()
    c_child_order = ProofEncryptionType_.c_child_order[:]
    c_cardinality = ProofEncryptionType_.c_cardinality.copy()

def proof_encryption_from_string(xml_string):
    return saml2.create_class_from_xml_string(ProofEncryption, xml_string)


class UseKey(UseKeyType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:UseKey element """

    c_tag = 'UseKey'
    c_namespace = NAMESPACE
    c_children = UseKeyType_.c_children.copy()
    c_attributes = UseKeyType_.c_attributes.copy()
    c_child_order = UseKeyType_.c_child_order[:]
    c_cardinality = UseKeyType_.c_cardinality.copy()

def use_key_from_string(xml_string):
    return saml2.create_class_from_xml_string(UseKey, xml_string)


class DelegateTo(DelegateToType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:DelegateTo element """

    c_tag = 'DelegateTo'
    c_namespace = NAMESPACE
    c_children = DelegateToType_.c_children.copy()
    c_attributes = DelegateToType_.c_attributes.copy()
    c_child_order = DelegateToType_.c_child_order[:]
    c_cardinality = DelegateToType_.c_cardinality.copy()

def delegate_to_from_string(xml_string):
    return saml2.create_class_from_xml_string(DelegateTo, xml_string)


class ParticipantsType_Primary(ParticipantType_):

    c_tag = 'Primary'
    c_namespace = NAMESPACE
    c_children = ParticipantType_.c_children.copy()
    c_attributes = ParticipantType_.c_attributes.copy()
    c_child_order = ParticipantType_.c_child_order[:]
    c_cardinality = ParticipantType_.c_cardinality.copy()

def participants_type__primary_from_string(xml_string):
    return saml2.create_class_from_xml_string(ParticipantsType_Primary, xml_string)


class ParticipantsType_Participant(ParticipantType_):

    c_tag = 'Participant'
    c_namespace = NAMESPACE
    c_children = ParticipantType_.c_children.copy()
    c_attributes = ParticipantType_.c_attributes.copy()
    c_child_order = ParticipantType_.c_child_order[:]
    c_cardinality = ParticipantType_.c_cardinality.copy()

def participants_type__participant_from_string(xml_string):
    return saml2.create_class_from_xml_string(ParticipantsType_Participant, xml_string)


class ParticipantsType_(SamlBase):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:ParticipantsType element """

    c_tag = 'ParticipantsType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}Primary'] = ('primary', ParticipantsType_Primary)
    c_cardinality['primary'] = {"min":0, "max":1}
    c_children['{http://docs.oasis-open.org/ws-sx/ws-trust/200512/}Participant'] = ('participant', [ParticipantsType_Participant])
    c_cardinality['participant'] = {"min":0}
    c_child_order.extend(['primary', 'participant'])

    def __init__(self,
            primary=None,
            participant=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.primary=primary
        self.participant=participant or []

def participants_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ParticipantsType_, xml_string)


class BinarySecret(BinarySecretType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:BinarySecret element """

    c_tag = 'BinarySecret'
    c_namespace = NAMESPACE
    c_children = BinarySecretType_.c_children.copy()
    c_attributes = BinarySecretType_.c_attributes.copy()
    c_child_order = BinarySecretType_.c_child_order[:]
    c_cardinality = BinarySecretType_.c_cardinality.copy()

def binary_secret_from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecret, xml_string)


class RequestSecurityTokenResponseCollection(RequestSecurityTokenResponseCollectionType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:RequestSecurityTokenResponseCollection element """

    c_tag = 'RequestSecurityTokenResponseCollection'
    c_namespace = NAMESPACE
    c_children = RequestSecurityTokenResponseCollectionType_.c_children.copy()
    c_attributes = RequestSecurityTokenResponseCollectionType_.c_attributes.copy()
    c_child_order = RequestSecurityTokenResponseCollectionType_.c_child_order[:]
    c_cardinality = RequestSecurityTokenResponseCollectionType_.c_cardinality.copy()

def request_security_token_response_collection_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestSecurityTokenResponseCollection, xml_string)


class Status(StatusType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Status element """

    c_tag = 'Status'
    c_namespace = NAMESPACE
    c_children = StatusType_.c_children.copy()
    c_attributes = StatusType_.c_attributes.copy()
    c_child_order = StatusType_.c_child_order[:]
    c_cardinality = StatusType_.c_cardinality.copy()

def status_from_string(xml_string):
    return saml2.create_class_from_xml_string(Status, xml_string)


class SignChallenge(SignChallengeType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:SignChallenge element """

    c_tag = 'SignChallenge'
    c_namespace = NAMESPACE
    c_children = SignChallengeType_.c_children.copy()
    c_attributes = SignChallengeType_.c_attributes.copy()
    c_child_order = SignChallengeType_.c_child_order[:]
    c_cardinality = SignChallengeType_.c_cardinality.copy()

def sign_challenge_from_string(xml_string):
    return saml2.create_class_from_xml_string(SignChallenge, xml_string)


class SignChallengeResponse(SignChallengeType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:SignChallengeResponse element """

    c_tag = 'SignChallengeResponse'
    c_namespace = NAMESPACE
    c_children = SignChallengeType_.c_children.copy()
    c_attributes = SignChallengeType_.c_attributes.copy()
    c_child_order = SignChallengeType_.c_child_order[:]
    c_cardinality = SignChallengeType_.c_cardinality.copy()

def sign_challenge_response_from_string(xml_string):
    return saml2.create_class_from_xml_string(SignChallengeResponse, xml_string)


class Authenticator(AuthenticatorType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Authenticator element """

    c_tag = 'Authenticator'
    c_namespace = NAMESPACE
    c_children = AuthenticatorType_.c_children.copy()
    c_attributes = AuthenticatorType_.c_attributes.copy()
    c_child_order = AuthenticatorType_.c_child_order[:]
    c_cardinality = AuthenticatorType_.c_cardinality.copy()

def authenticator_from_string(xml_string):
    return saml2.create_class_from_xml_string(Authenticator, xml_string)


class Participants(ParticipantsType_):
    """The http://docs.oasis-open.org/ws-sx/ws-trust/200512/:Participants element """

    c_tag = 'Participants'
    c_namespace = NAMESPACE
    c_children = ParticipantsType_.c_children.copy()
    c_attributes = ParticipantsType_.c_attributes.copy()
    c_child_order = ParticipantsType_.c_child_order[:]
    c_cardinality = ParticipantsType_.c_cardinality.copy()

def participants_from_string(xml_string):
    return saml2.create_class_from_xml_string(Participants, xml_string)


ELEMENT_FROM_STRING = {
    RequestSecurityToken.c_tag: request_security_token_from_string,
    RequestSecurityTokenType_.c_tag: request_security_token_type__from_string,
    TokenType.c_tag: token_type_from_string,
    RequestType.c_tag: request_type_from_string,
    RequestTypeOpenEnum_.c_tag: request_type_open_enum__from_string,
    RequestTypeEnum_.c_tag: request_type_enum__from_string,
    RequestSecurityTokenResponse.c_tag: request_security_token_response_from_string,
    RequestSecurityTokenResponseType_.c_tag: request_security_token_response_type__from_string,
    RequestedSecurityToken.c_tag: requested_security_token_from_string,
    RequestedSecurityTokenType_.c_tag: requested_security_token_type__from_string,
    BinarySecret.c_tag: binary_secret_from_string,
    BinarySecretType_.c_tag: binary_secret_type__from_string,
    BinarySecretTypeEnum_.c_tag: binary_secret_type_enum__from_string,
    BinarySecretTypeOpenEnum_.c_tag: binary_secret_type_open_enum__from_string,
    Claims.c_tag: claims_from_string,
    ClaimsType_.c_tag: claims_type__from_string,
    Entropy.c_tag: entropy_from_string,
    EntropyType_.c_tag: entropy_type__from_string,
    Lifetime.c_tag: lifetime_from_string,
    LifetimeType_.c_tag: lifetime_type__from_string,
    RequestSecurityTokenCollection.c_tag: request_security_token_collection_from_string,
    RequestSecurityTokenCollectionType_.c_tag: request_security_token_collection_type__from_string,
    RequestSecurityTokenResponseCollection.c_tag: request_security_token_response_collection_from_string,
    RequestSecurityTokenResponseCollectionType_.c_tag: request_security_token_response_collection_type__from_string,
    ComputedKey.c_tag: computed_key_from_string,
    ComputedKeyEnum_.c_tag: computed_key_enum__from_string,
    ComputedKeyOpenEnum_.c_tag: computed_key_open_enum__from_string,
    RequestedAttachedReference.c_tag: requested_attached_reference_from_string,
    RequestedUnattachedReference.c_tag: requested_unattached_reference_from_string,
    RequestedReferenceType_.c_tag: requested_reference_type__from_string,
    RequestedProofToken.c_tag: requested_proof_token_from_string,
    RequestedProofTokenType_.c_tag: requested_proof_token_type__from_string,
    IssuedTokens.c_tag: issued_tokens_from_string,
    RenewTarget.c_tag: renew_target_from_string,
    RenewTargetType_.c_tag: renew_target_type__from_string,
    AllowPostdating.c_tag: allow_postdating_from_string,
    AllowPostdatingType_.c_tag: allow_postdating_type__from_string,
    Renewing.c_tag: renewing_from_string,
    RenewingType_.c_tag: renewing_type__from_string,
    CancelTarget.c_tag: cancel_target_from_string,
    CancelTargetType_.c_tag: cancel_target_type__from_string,
    RequestedTokenCancelled.c_tag: requested_token_cancelled_from_string,
    RequestedTokenCancelledType_.c_tag: requested_token_cancelled_type__from_string,
    ValidateTarget.c_tag: validate_target_from_string,
    ValidateTargetType_.c_tag: validate_target_type__from_string,
    Status.c_tag: status_from_string,
    StatusType_.c_tag: status_type__from_string,
    StatusCodeEnum_.c_tag: status_code_enum__from_string,
    StatusCodeOpenEnum_.c_tag: status_code_open_enum__from_string,
    SignChallenge.c_tag: sign_challenge_from_string,
    SignChallengeResponse.c_tag: sign_challenge_response_from_string,
    SignChallengeType_.c_tag: sign_challenge_type__from_string,
    Challenge.c_tag: challenge_from_string,
    BinaryExchange.c_tag: binary_exchange_from_string,
    BinaryExchangeType_.c_tag: binary_exchange_type__from_string,
    RequestKET.c_tag: request_ket_from_string,
    RequestKETType_.c_tag: request_ket_type__from_string,
    KeyExchangeToken.c_tag: key_exchange_token_from_string,
    KeyExchangeTokenType_.c_tag: key_exchange_token_type__from_string,
    Authenticator.c_tag: authenticator_from_string,
    AuthenticatorType_.c_tag: authenticator_type__from_string,
    CombinedHash.c_tag: combined_hash_from_string,
    OnBehalfOf.c_tag: on_behalf_of_from_string,
    OnBehalfOfType_.c_tag: on_behalf_of_type__from_string,
    Issuer.c_tag: issuer_from_string,
    AuthenticationType.c_tag: authentication_type_from_string,
    KeyType.c_tag: key_type_from_string,
    KeyTypeEnum_.c_tag: key_type_enum__from_string,
    KeyTypeOpenEnum_.c_tag: key_type_open_enum__from_string,
    KeySize.c_tag: key_size_from_string,
    SignatureAlgorithm.c_tag: signature_algorithm_from_string,
    EncryptionAlgorithm.c_tag: encryption_algorithm_from_string,
    CanonicalizationAlgorithm.c_tag: canonicalization_algorithm_from_string,
    ComputedKeyAlgorithm.c_tag: computed_key_algorithm_from_string,
    Encryption.c_tag: encryption_from_string,
    EncryptionType_.c_tag: encryption_type__from_string,
    ProofEncryption.c_tag: proof_encryption_from_string,
    ProofEncryptionType_.c_tag: proof_encryption_type__from_string,
    UseKey.c_tag: use_key_from_string,
    UseKeyType_.c_tag: use_key_type__from_string,
    KeyWrapAlgorithm.c_tag: key_wrap_algorithm_from_string,
    SignWith.c_tag: sign_with_from_string,
    EncryptWith.c_tag: encrypt_with_from_string,
    DelegateTo.c_tag: delegate_to_from_string,
    DelegateToType_.c_tag: delegate_to_type__from_string,
    Forwardable.c_tag: forwardable_from_string,
    Delegatable.c_tag: delegatable_from_string,
    Participants.c_tag: participants_from_string,
    ParticipantsType_.c_tag: participants_type__from_string,
    ParticipantType_.c_tag: participant_type__from_string,
    StatusType_Code.c_tag: status_type__code_from_string,
    StatusType_Reason.c_tag: status_type__reason_from_string,
    ParticipantsType_Primary.c_tag: participants_type__primary_from_string,
    ParticipantsType_Participant.c_tag: participants_type__participant_from_string,
}

ELEMENT_BY_TAG = {
    'RequestSecurityToken': RequestSecurityToken,
    'RequestSecurityTokenType': RequestSecurityTokenType_,
    'TokenType': TokenType,
    'RequestType': RequestType,
    'RequestTypeOpenEnum': RequestTypeOpenEnum_,
    'RequestTypeEnum': RequestTypeEnum_,
    'RequestSecurityTokenResponse': RequestSecurityTokenResponse,
    'RequestSecurityTokenResponseType': RequestSecurityTokenResponseType_,
    'RequestedSecurityToken': RequestedSecurityToken,
    'RequestedSecurityTokenType': RequestedSecurityTokenType_,
    'BinarySecret': BinarySecret,
    'BinarySecretType': BinarySecretType_,
    'BinarySecretTypeEnum': BinarySecretTypeEnum_,
    'BinarySecretTypeOpenEnum': BinarySecretTypeOpenEnum_,
    'Claims': Claims,
    'ClaimsType': ClaimsType_,
    'Entropy': Entropy,
    'EntropyType': EntropyType_,
    'Lifetime': Lifetime,
    'LifetimeType': LifetimeType_,
    'RequestSecurityTokenCollection': RequestSecurityTokenCollection,
    'RequestSecurityTokenCollectionType': RequestSecurityTokenCollectionType_,
    'RequestSecurityTokenResponseCollection': RequestSecurityTokenResponseCollection,
    'RequestSecurityTokenResponseCollectionType': RequestSecurityTokenResponseCollectionType_,
    'ComputedKey': ComputedKey,
    'ComputedKeyEnum': ComputedKeyEnum_,
    'ComputedKeyOpenEnum': ComputedKeyOpenEnum_,
    'RequestedAttachedReference': RequestedAttachedReference,
    'RequestedUnattachedReference': RequestedUnattachedReference,
    'RequestedReferenceType': RequestedReferenceType_,
    'RequestedProofToken': RequestedProofToken,
    'RequestedProofTokenType': RequestedProofTokenType_,
    'IssuedTokens': IssuedTokens,
    'RenewTarget': RenewTarget,
    'RenewTargetType': RenewTargetType_,
    'AllowPostdating': AllowPostdating,
    'AllowPostdatingType': AllowPostdatingType_,
    'Renewing': Renewing,
    'RenewingType': RenewingType_,
    'CancelTarget': CancelTarget,
    'CancelTargetType': CancelTargetType_,
    'RequestedTokenCancelled': RequestedTokenCancelled,
    'RequestedTokenCancelledType': RequestedTokenCancelledType_,
    'ValidateTarget': ValidateTarget,
    'ValidateTargetType': ValidateTargetType_,
    'Status': Status,
    'StatusType': StatusType_,
    'StatusCodeEnum': StatusCodeEnum_,
    'StatusCodeOpenEnum': StatusCodeOpenEnum_,
    'SignChallenge': SignChallenge,
    'SignChallengeResponse': SignChallengeResponse,
    'SignChallengeType': SignChallengeType_,
    'Challenge': Challenge,
    'BinaryExchange': BinaryExchange,
    'BinaryExchangeType': BinaryExchangeType_,
    'RequestKET': RequestKET,
    'RequestKETType': RequestKETType_,
    'KeyExchangeToken': KeyExchangeToken,
    'KeyExchangeTokenType': KeyExchangeTokenType_,
    'Authenticator': Authenticator,
    'AuthenticatorType': AuthenticatorType_,
    'CombinedHash': CombinedHash,
    'OnBehalfOf': OnBehalfOf,
    'OnBehalfOfType': OnBehalfOfType_,
    'Issuer': Issuer,
    'AuthenticationType': AuthenticationType,
    'KeyType': KeyType,
    'KeyTypeEnum': KeyTypeEnum_,
    'KeyTypeOpenEnum': KeyTypeOpenEnum_,
    'KeySize': KeySize,
    'SignatureAlgorithm': SignatureAlgorithm,
    'EncryptionAlgorithm': EncryptionAlgorithm,
    'CanonicalizationAlgorithm': CanonicalizationAlgorithm,
    'ComputedKeyAlgorithm': ComputedKeyAlgorithm,
    'Encryption': Encryption,
    'EncryptionType': EncryptionType_,
    'ProofEncryption': ProofEncryption,
    'ProofEncryptionType': ProofEncryptionType_,
    'UseKey': UseKey,
    'UseKeyType': UseKeyType_,
    'KeyWrapAlgorithm': KeyWrapAlgorithm,
    'SignWith': SignWith,
    'EncryptWith': EncryptWith,
    'DelegateTo': DelegateTo,
    'DelegateToType': DelegateToType_,
    'Forwardable': Forwardable,
    'Delegatable': Delegatable,
    'Participants': Participants,
    'ParticipantsType': ParticipantsType_,
    'ParticipantType': ParticipantType_,
    'Code': StatusType_Code,
    'Reason': StatusType_Reason,
    'Primary': ParticipantsType_Primary,
    'Participant': ParticipantsType_Participant,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)




#!/usr/bin/env python

#
# Generated Sun Jun 14 13:29:15 2015 by parse_xsd.py version 0.5.
#

import saml2
from saml2 import SamlBase

from saml2 import xmldsig as ds
from saml2.schema import soapenv
from saml2.ws import wsutil as wsu

NAMESPACE = 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd'

class AttributedString_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:AttributedString element """

    c_tag = 'AttributedString'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id'] = ('Id', 'string', False)

    def __init__(self,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.Id=Id

def attributed_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(AttributedString_, xml_string)

class PasswordString_(AttributedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:PasswordString element """

    c_tag = 'PasswordString'
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()
    c_attributes['Type'] = ('type', 'anyURI', False)

    def __init__(self,
            type=None,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        AttributedString_.__init__(self, 
                Id=Id,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.type=type

def password_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(PasswordString_, xml_string)

class EncodedString_(AttributedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:EncodedString element """

    c_tag = 'EncodedString'
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()
    c_attributes['EncodingType'] = ('encoding_type', 'anyURI', False)

    def __init__(self,
            encoding_type=None,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        AttributedString_.__init__(self, 
                Id=Id,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.encoding_type=encoding_type

def encoded_string__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncodedString_, xml_string)

class UsernameTokenType_Username(AttributedString_):

    c_tag = 'Username'
    c_namespace = NAMESPACE
    c_children = AttributedString_.c_children.copy()
    c_attributes = AttributedString_.c_attributes.copy()
    c_child_order = AttributedString_.c_child_order[:]
    c_cardinality = AttributedString_.c_cardinality.copy()

def username_token_type__username_from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameTokenType_Username, xml_string)

class UsernameTokenType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:UsernameTokenType element """

    c_tag = 'UsernameTokenType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd}Username'] = ('username', UsernameTokenType_Username)
    c_attributes['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id'] = ('Id', 'None', False)
    c_child_order.extend(['username'])

    def __init__(self,
            username=None,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.username=username
        self.Id=Id

def username_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameTokenType_, xml_string)

class BinarySecurityTokenType_(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:BinarySecurityTokenType element """

    c_tag = 'BinarySecurityTokenType'
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()
    c_attributes['ValueType'] = ('value_type', 'anyURI', False)

    def __init__(self,
            value_type=None,
            encoding_type=None,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        EncodedString_.__init__(self, 
                encoding_type=encoding_type,
                Id=Id,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.value_type=value_type

def binary_security_token_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecurityTokenType_, xml_string)

class KeyIdentifierType_(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:KeyIdentifierType element """

    c_tag = 'KeyIdentifierType'
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()
    c_attributes['ValueType'] = ('value_type', 'anyURI', False)

    def __init__(self,
            value_type=None,
            encoding_type=None,
            Id=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        EncodedString_.__init__(self, 
                encoding_type=encoding_type,
                Id=Id,
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.value_type=value_type

def key_identifier_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyIdentifierType_, xml_string)

class TUsage_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:tUsage element """

    c_tag = 'tUsage'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def t_usage__from_string(xml_string):
    return saml2.create_class_from_xml_string(TUsage_, xml_string)

class ReferenceType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:ReferenceType element """

    c_tag = 'ReferenceType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['URI'] = ('uri', 'anyURI', False)
    c_attributes['ValueType'] = ('value_type', 'anyURI', False)

    def __init__(self,
            uri=None,
            value_type=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.uri=uri
        self.value_type=value_type

def reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(ReferenceType_, xml_string)

class EmbeddedType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:EmbeddedType element """

    c_tag = 'EmbeddedType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['ValueType'] = ('value_type', 'anyURI', False)

    def __init__(self,
            value_type=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.value_type=value_type

def embedded_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(EmbeddedType_, xml_string)

class SecurityTokenReferenceType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityTokenReferenceType element """

    c_tag = 'SecurityTokenReferenceType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['{http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd}Id'] = ('Id', 'None', False)
    c_attributes['Usage'] = ('Usage', 'None', False)

    def __init__(self,
            Id=None,
            Usage=None,
            text=None,
            extension_elements=None,
            extension_attributes=None,
        ):
        SamlBase.__init__(self, 
                text=text,
                extension_elements=extension_elements,
                extension_attributes=extension_attributes,
                )
        self.Id=Id
        self.Usage=Usage

def security_token_reference_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityTokenReferenceType_, xml_string)

class SecurityHeaderType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityHeaderType element """

    c_tag = 'SecurityHeaderType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def security_header_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityHeaderType_, xml_string)

class TransformationParametersType_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:TransformationParametersType element """

    c_tag = 'TransformationParametersType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def transformation_parameters_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformationParametersType_, xml_string)

class UsernameToken(UsernameTokenType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:UsernameToken element """

    c_tag = 'UsernameToken'
    c_namespace = NAMESPACE
    c_children = UsernameTokenType_.c_children.copy()
    c_attributes = UsernameTokenType_.c_attributes.copy()
    c_child_order = UsernameTokenType_.c_child_order[:]
    c_cardinality = UsernameTokenType_.c_cardinality.copy()

def username_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(UsernameToken, xml_string)

class BinarySecurityToken(BinarySecurityTokenType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:BinarySecurityToken element """

    c_tag = 'BinarySecurityToken'
    c_namespace = NAMESPACE
    c_children = BinarySecurityTokenType_.c_children.copy()
    c_attributes = BinarySecurityTokenType_.c_attributes.copy()
    c_child_order = BinarySecurityTokenType_.c_child_order[:]
    c_cardinality = BinarySecurityTokenType_.c_cardinality.copy()

def binary_security_token_from_string(xml_string):
    return saml2.create_class_from_xml_string(BinarySecurityToken, xml_string)


class Reference(ReferenceType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Reference element """

    c_tag = 'Reference'
    c_namespace = NAMESPACE
    c_children = ReferenceType_.c_children.copy()
    c_attributes = ReferenceType_.c_attributes.copy()
    c_child_order = ReferenceType_.c_child_order[:]
    c_cardinality = ReferenceType_.c_cardinality.copy()

def reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(Reference, xml_string)


class Embedded(EmbeddedType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Embedded element """

    c_tag = 'Embedded'
    c_namespace = NAMESPACE
    c_children = EmbeddedType_.c_children.copy()
    c_attributes = EmbeddedType_.c_attributes.copy()
    c_child_order = EmbeddedType_.c_child_order[:]
    c_cardinality = EmbeddedType_.c_cardinality.copy()

def embedded_from_string(xml_string):
    return saml2.create_class_from_xml_string(Embedded, xml_string)


class KeyIdentifier(KeyIdentifierType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:KeyIdentifier element """

    c_tag = 'KeyIdentifier'
    c_namespace = NAMESPACE
    c_children = KeyIdentifierType_.c_children.copy()
    c_attributes = KeyIdentifierType_.c_attributes.copy()
    c_child_order = KeyIdentifierType_.c_child_order[:]
    c_cardinality = KeyIdentifierType_.c_cardinality.copy()

def key_identifier_from_string(xml_string):
    return saml2.create_class_from_xml_string(KeyIdentifier, xml_string)


class SecurityTokenReference(SecurityTokenReferenceType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:SecurityTokenReference element """

    c_tag = 'SecurityTokenReference'
    c_namespace = NAMESPACE
    c_children = SecurityTokenReferenceType_.c_children.copy()
    c_attributes = SecurityTokenReferenceType_.c_attributes.copy()
    c_child_order = SecurityTokenReferenceType_.c_child_order[:]
    c_cardinality = SecurityTokenReferenceType_.c_cardinality.copy()

def security_token_reference_from_string(xml_string):
    return saml2.create_class_from_xml_string(SecurityTokenReference, xml_string)


class Security(SecurityHeaderType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Security element """

    c_tag = 'Security'
    c_namespace = NAMESPACE
    c_children = SecurityHeaderType_.c_children.copy()
    c_attributes = SecurityHeaderType_.c_attributes.copy()
    c_child_order = SecurityHeaderType_.c_child_order[:]
    c_cardinality = SecurityHeaderType_.c_cardinality.copy()

def security_from_string(xml_string):
    return saml2.create_class_from_xml_string(Security, xml_string)


class TransformationParameters(TransformationParametersType_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:TransformationParameters element """

    c_tag = 'TransformationParameters'
    c_namespace = NAMESPACE
    c_children = TransformationParametersType_.c_children.copy()
    c_attributes = TransformationParametersType_.c_attributes.copy()
    c_child_order = TransformationParametersType_.c_child_order[:]
    c_cardinality = TransformationParametersType_.c_cardinality.copy()

def transformation_parameters_from_string(xml_string):
    return saml2.create_class_from_xml_string(TransformationParameters, xml_string)


class Password(PasswordString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Password element """

    c_tag = 'Password'
    c_namespace = NAMESPACE
    c_children = PasswordString_.c_children.copy()
    c_attributes = PasswordString_.c_attributes.copy()
    c_child_order = PasswordString_.c_child_order[:]
    c_cardinality = PasswordString_.c_cardinality.copy()

def password_from_string(xml_string):
    return saml2.create_class_from_xml_string(Password, xml_string)


class Nonce(EncodedString_):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:Nonce element """

    c_tag = 'Nonce'
    c_namespace = NAMESPACE
    c_children = EncodedString_.c_children.copy()
    c_attributes = EncodedString_.c_attributes.copy()
    c_child_order = EncodedString_.c_child_order[:]
    c_cardinality = EncodedString_.c_cardinality.copy()

def nonce_from_string(xml_string):
    return saml2.create_class_from_xml_string(Nonce, xml_string)


class FaultcodeEnum_(SamlBase):
    """The http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd:FaultcodeEnum element """

    c_tag = 'FaultcodeEnum'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'xsd:QName', 'enumeration': ['wsse:UnsupportedSecurityToken', 'wsse:UnsupportedAlgorithm', 'wsse:InvalidSecurity', 'wsse:InvalidSecurityToken', 'wsse:FailedAuthentication', 'wsse:FailedCheck', 'wsse:SecurityTokenUnavailable']}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()

def faultcode_enum__from_string(xml_string):
    return saml2.create_class_from_xml_string(FaultcodeEnum_, xml_string)


#..................
('#', [])
ELEMENT_FROM_STRING = {
    AttributedString_.c_tag: attributed_string__from_string,
    PasswordString_.c_tag: password_string__from_string,
    EncodedString_.c_tag: encoded_string__from_string,
    UsernameTokenType_.c_tag: username_token_type__from_string,
    BinarySecurityTokenType_.c_tag: binary_security_token_type__from_string,
    KeyIdentifierType_.c_tag: key_identifier_type__from_string,
    TUsage_.c_tag: t_usage__from_string,
    ReferenceType_.c_tag: reference_type__from_string,
    EmbeddedType_.c_tag: embedded_type__from_string,
    SecurityTokenReferenceType_.c_tag: security_token_reference_type__from_string,
    SecurityHeaderType_.c_tag: security_header_type__from_string,
    TransformationParametersType_.c_tag: transformation_parameters_type__from_string,
    UsernameToken.c_tag: username_token_from_string,
    BinarySecurityToken.c_tag: binary_security_token_from_string,
    Reference.c_tag: reference_from_string,
    Embedded.c_tag: embedded_from_string,
    KeyIdentifier.c_tag: key_identifier_from_string,
    SecurityTokenReference.c_tag: security_token_reference_from_string,
    Security.c_tag: security_from_string,
    TransformationParameters.c_tag: transformation_parameters_from_string,
    Password.c_tag: password_from_string,
    Nonce.c_tag: nonce_from_string,
    FaultcodeEnum_.c_tag: faultcode_enum__from_string,
    UsernameTokenType_Username.c_tag: username_token_type__username_from_string,
}

ELEMENT_BY_TAG = {
    'AttributedString': AttributedString_,
    'PasswordString': PasswordString_,
    'EncodedString': EncodedString_,
    'UsernameTokenType': UsernameTokenType_,
    'BinarySecurityTokenType': BinarySecurityTokenType_,
    'KeyIdentifierType': KeyIdentifierType_,
    'tUsage': TUsage_,
    'ReferenceType': ReferenceType_,
    'EmbeddedType': EmbeddedType_,
    'SecurityTokenReferenceType': SecurityTokenReferenceType_,
    'SecurityHeaderType': SecurityHeaderType_,
    'TransformationParametersType': TransformationParametersType_,
    'UsernameToken': UsernameToken,
    'BinarySecurityToken': BinarySecurityToken,
    'Reference': Reference,
    'Embedded': Embedded,
    'KeyIdentifier': KeyIdentifier,
    'SecurityTokenReference': SecurityTokenReference,
    'Security': Security,
    'TransformationParameters': TransformationParameters,
    'Password': Password,
    'Nonce': Nonce,
    'FaultcodeEnum': FaultcodeEnum_,
    'Username': UsernameTokenType_Username,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)




from six.moves.urllib.parse import urlencode

from paste.httpheaders import CONTENT_LENGTH
from paste.httpheaders import CONTENT_TYPE
from paste.httpheaders import LOCATION
from paste.httpexceptions import HTTPFound

from paste.request import parse_dict_querystring
from paste.request import parse_formvars
from paste.request import construct_url

from zope.interface import implements

from repoze.who.interfaces import IChallenger
from repoze.who.interfaces import IIdentifier
from repoze.who.plugins.form import FormPlugin

_DEFAULT_FORM = """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" >
<html>
<head>
    <title>Demo Organization Log In</title>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
</head>
<body>
    <div>
        <strong>Demo Organization Log In</strong>
    </div>
    <form name="foo" method=POST action="?__do_login=true">
        <input type="text" name="login">
        <input type="password" name="password">
        <input name="submit" type="submit" value="Logga in">
        %s
    </form>
</body>
</html>
"""

HIDDEN_PRE_LINE = """<input type=hidden name="%s" value="%s">"""


class FormHiddenPlugin(FormPlugin):

    implements(IChallenger, IIdentifier)

    # IIdentifier
    def identify(self, environ):
        logger = environ.get("repoze.who.logger", "")
        logger.info("formplugin identify")
        # logger and logger.info("environ keys: %s", environ.keys())
        query = parse_dict_querystring(environ)
        # If the extractor finds a special query string on any request,
        # it will attempt to find the values in the input body.
        if query.get(self.login_form_qs):
            form = parse_formvars(environ)
            from StringIO import StringIO

            # we need to replace wsgi.input because we've read it
            # this smells funny
            environ["wsgi.input"] = StringIO()
            form.update(query)
            qinfo = {}
            for key, val in form.items():
                if key.startswith("_") and key.endswith("_"):
                    qinfo[key[1:-1]] = val
            if qinfo:
                environ["s2repoze.qinfo"] = qinfo
            try:
                login = form["login"]
                password = form["password"]
            except KeyError:
                return None
            del query[self.login_form_qs]
            query.update(qinfo)
            environ["QUERY_STRING"] = urlencode(query)
            environ["repoze.who.application"] = HTTPFound(construct_url(environ))
            credentials = {"login": login, "password": password}
            max_age = form.get("max_age", None)
            if max_age is not None:
                credentials["max_age"] = max_age
            return credentials

        return None

    # IChallenger
    def challenge(self, environ, status, app_headers, forget_headers):
        logger = environ.get("repoze.who.logger", "")
        logger.info("formplugin challenge")
        if app_headers:
            location = LOCATION(app_headers)
            if location:
                headers = list(app_headers) + list(forget_headers)
                return HTTPFound(headers=headers)

        query = parse_dict_querystring(environ)
        hidden = []
        for key, val in query.items():
            hidden.append(HIDDEN_PRE_LINE % ("_%s_" % key, val))

        logger.info("hidden: %s", hidden)
        form = self.formbody or _DEFAULT_FORM
        form = form % "\n".join(hidden)

        if self.formcallable is not None:
            form = self.formcallable(environ)

        def auth_form(environ, start_response):
            content_length = CONTENT_LENGTH.tuples(str(len(form)))
            content_type = CONTENT_TYPE.tuples("text/html")
            headers = content_length + content_type + forget_headers
            start_response("200 OK", headers)
            return [form]

        return auth_form


def make_plugin(login_form_qs="__do_login", rememberer_name=None, form=None):
    if rememberer_name is None:
        raise ValueError(
            "must include rememberer key (name of another IIdentifier plugin)"
        )
    if form is not None:
        with open(form, "r") as f:
            form = f.read()
    plugin = FormHiddenPlugin(login_form_qs, rememberer_name, form)
    return plugin



import logging
import six
import time
from saml2 import SAMLError
import saml2.cryptography.symmetric
from saml2.httputil import Response
from saml2.httputil import make_cookie
from saml2.httputil import Redirect
from saml2.httputil import Unauthorized
from saml2.httputil import parse_cookie

from six.moves.urllib.parse import urlencode, parse_qs, urlsplit

__author__ = 'rolandh'

logger = logging.getLogger(__name__)


class AuthnFailure(SAMLError):
    pass


class EncodeError(SAMLError):
    pass


class UserAuthnMethod(object):
    def __init__(self, srv):
        self.srv = srv

    def __call__(self, *args, **kwargs):
        raise NotImplementedError

    def authenticated_as(self, **kwargs):
        raise NotImplementedError

    def verify(self, **kwargs):
        raise NotImplementedError


def is_equal(a, b):
    if len(a) != len(b):
        return False

    result = 0
    for x, y in zip(a, b):
        result |= x ^ y
    return result == 0


def url_encode_params(params=None):
    if not isinstance(params, dict):
        raise EncodeError("You must pass in a dictionary!")
    params_list = []
    for k, v in params.items():
        if isinstance(v, list):
            params_list.extend([(k, x) for x in v])
        else:
            params_list.append((k, v))
    return urlencode(params_list)


def create_return_url(base, query, **kwargs):
    """
    Add a query string plus extra parameters to a base URL which may contain
    a query part already.

    :param base: redirect_uri may contain a query part, no fragment allowed.
    :param query: Old query part as a string
    :param kwargs: extra query parameters
    :return:
    """
    part = urlsplit(base)
    if part.fragment:
        raise ValueError("Base URL contained parts it shouldn't")

    for key, values in parse_qs(query).items():
        if key in kwargs:
            if isinstance(kwargs[key], six.string_types):
                kwargs[key] = [kwargs[key]]
            kwargs[key].extend(values)
        else:
            kwargs[key] = values

    if part.query:
        for key, values in parse_qs(part.query).items():
            if key in kwargs:
                if isinstance(kwargs[key], six.string_types):
                    kwargs[key] = [kwargs[key]]
                kwargs[key].extend(values)
            else:
                kwargs[key] = values

        _pre = base.split("?")[0]
    else:
        _pre = base

    logger.debug("kwargs: %s" % kwargs)

    return "%s?%s" % (_pre, url_encode_params(kwargs))


class UsernamePasswordMako(UserAuthnMethod):
    """Do user authentication using the normal username password form
    using Mako as template system"""
    cookie_name = "userpassmako"

    def __init__(self, srv, mako_template, template_lookup, pwd, return_to):
        """
        :param srv: The server instance
        :param mako_template: Which Mako template to use
        :param pwd: Username/password dictionary like database
        :param return_to: Where to send the user after authentication
        :return:
        """
        UserAuthnMethod.__init__(self, srv)
        self.mako_template = mako_template
        self.template_lookup = template_lookup
        self.passwd = pwd
        self.return_to = return_to
        self.active = {}
        self.query_param = "upm_answer"
        self.symmetric = saml2.cryptography.symmetric.Default(srv.symkey)

    def __call__(self, cookie=None, policy_url=None, logo_url=None,
                 query="", **kwargs):
        """
        Put up the login form
        """
        if cookie:
            headers = [cookie]
        else:
            headers = []

        resp = Response(headers=headers)

        argv = {"login": "",
                "password": "",
                "action": "verify",
                "policy_url": policy_url,
                "logo_url": logo_url,
                "query": query}
        logger.info("do_authentication argv: %s" % argv)
        mte = self.template_lookup.get_template(self.mako_template)
        resp.message = mte.render(**argv)
        return resp

    def _verify(self, pwd, user):
        if not is_equal(pwd, self.passwd[user]):
            raise ValueError("Wrong password")

    def verify(self, request, **kwargs):
        """
        Verifies that the given username and password was correct
        :param request: Either the query part of a URL a urlencoded
            body of a HTTP message or a parse such.
        :param kwargs: Catch whatever else is sent.
        :return: redirect back to where ever the base applications
            wants the user after authentication.
        """

        #logger.debug("verify(%s)" % request)
        if isinstance(request, six.string_types):
            _dict = parse_qs(request)
        elif isinstance(request, dict):
            _dict = request
        else:
            raise ValueError("Wrong type of input")

        # verify username and password
        try:
            self._verify(_dict["password"][0], _dict["login"][0])
            timestamp = str(int(time.mktime(time.gmtime())))
            msg = "::".join([_dict["login"][0], timestamp])
            info = self.symmetric.encrypt(msg.encode())
            self.active[info] = timestamp
            cookie = make_cookie(self.cookie_name, info, self.srv.seed)
            return_to = create_return_url(self.return_to, _dict["query"][0],
                                          **{self.query_param: "true"})
            resp = Redirect(return_to, headers=[cookie])
        except (ValueError, KeyError):
            resp = Unauthorized("Unknown user or wrong password")

        return resp

    def authenticated_as(self, cookie=None, **kwargs):
        if cookie is None:
            return None
        else:
            logger.debug("kwargs: %s" % kwargs)
            try:
                info, timestamp = parse_cookie(self.cookie_name,
                                               self.srv.seed, cookie)
                if self.active[info] == timestamp:
                    msg = self.symmetric.decrypt(info).decode()
                    uid, _ts = msg.split("::")
                    if timestamp == _ts:
                        return {"uid": uid}
            except Exception:
                pass

        return None

    def done(self, areq):
        try:
            _ = areq[self.query_param]
            return False
        except KeyError:
            return True


class SocialService(UserAuthnMethod):
    def __init__(self, social):
        UserAuthnMethod.__init__(self, None)
        self.social = social

    def __call__(self, server_env, cookie=None, sid="", query="", **kwargs):
        return self.social.begin(server_env, cookie, sid, query)

    def callback(self, server_env, cookie=None, sid="", query="", **kwargs):
        return self.social.callback(server_env, cookie, sid, query, **kwargs)


class AuthnMethodChooser(object):
    def __init__(self, methods=None):
        self.methods = methods

    def __call__(self, **kwargs):
        if not self.methods:
            raise SAMLError("No authentication methods defined")
        elif len(self.methods) == 1:
            return self.methods[0]
        else:
            pass  # TODO

try:
    import ldap

    class LDAPAuthn(UsernamePasswordMako):
        def __init__(self, srv, ldapsrv, return_to,
                     dn_pattern, mako_template, template_lookup):
            """
            :param srv: The server instance
            :param ldapsrv: Which LDAP server to us
            :param return_to: Where to send the user after authentication
            :return:
            """
            UsernamePasswordMako.__init__(self, srv, mako_template, template_lookup,
                                          None, return_to)

            self.ldap = ldap.initialize(ldapsrv)
            self.ldap.protocol_version = 3
            self.ldap.set_option(ldap.OPT_REFERRALS, 0)
            self.dn_pattern = dn_pattern

        def _verify(self, pwd, user):
            """
            Verifies the username and password agains a LDAP server
            :param pwd: The password
            :param user: The username
            :return: AssertionError if the LDAP verification failed.
            """
            _dn = self.dn_pattern % user
            try:
                self.ldap.simple_bind_s(_dn, pwd)
            except Exception:
                raise AssertionError()
except ImportError:
    class LDAPAuthn(UserAuthnMethod):
        pass



import pkg_resources as _pkg_resources


def _parse_version():
    data = _pkg_resources.get_distribution('pysaml2')
    value = _pkg_resources.parse_version(data.version)
    return value


version_info = _parse_version()
version = str(version_info)



# Interface to external user info resources

import copy


class UserInfo(object):
    """ Read only interface to a user info store """

    def __init__(self):
        pass

    def __call__(self, **kwargs):
        pass


class UserInfoDB(UserInfo):
    """ Read only interface to a user info store """

    def __init__(self, db=None):
        self.db = db

    @staticmethod
    def filter(userinfo, user_info_claims=None):
        """
        Return only those claims that are asked for.
        It's a best effort task; if essential claims are not present
        no error is flagged.

        :param userinfo: A dictionary containing the available user info.
        :param user_info_claims: A dictionary specifying the asked for claims
        :return: A dictionary of filtered claims.
        """

        if user_info_claims is None:
            return copy.copy(userinfo)
        else:
            result = {}
            missing = []
            optional = []
            for key, restr in user_info_claims.items():
                try:
                    result[key] = userinfo[key]
                except KeyError:
                    if restr == {"essential": True}:
                        missing.append(key)
                    else:
                        optional.append(key)
            return result

    def __call__(self, userid, user_info_claims=None, **kwargs):
        try:
            return self.filter(self.db[userid], user_info_claims)
        except KeyError:
            return {}




#!/usr/bin/env python

#
# Generated Thu May 15 13:58:36 2014 by parse_xsd.py version 0.5.
#

import saml2

from saml2 import md

NAMESPACE = 'urn:oasis:names:tc:SAML:profiles:SSO:request-init'


class RequestInitiator(md.EndpointType_):
    """The urn:oasis:names:tc:SAML:profiles:SSO:request-init:RequestInitiator
    element """

    c_tag = 'RequestInitiator'
    c_namespace = NAMESPACE
    c_children = md.EndpointType_.c_children.copy()
    c_attributes = md.EndpointType_.c_attributes.copy()
    c_child_order = md.EndpointType_.c_child_order[:]
    c_cardinality = md.EndpointType_.c_cardinality.copy()


def request_initiator_from_string(xml_string):
    return saml2.create_class_from_xml_string(RequestInitiator, xml_string)


ELEMENT_FROM_STRING = {
    RequestInitiator.c_tag: request_initiator_from_string,
}

ELEMENT_BY_TAG = {
    'RequestInitiator': RequestInitiator,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)




import copy
import importlib
import logging
import logging.handlers
import os
import re
import sys
from logging.config import dictConfig as configure_logging_by_dict
from warnings import warn as _warn

import six

from saml2 import BINDING_HTTP_ARTIFACT
from saml2 import BINDING_HTTP_POST
from saml2 import BINDING_HTTP_REDIRECT
from saml2 import BINDING_SOAP
from saml2 import BINDING_URI
from saml2 import SAMLError

from saml2.attribute_converter import ac_factory
from saml2.assertion import Policy
from saml2.mdstore import MetadataStore
from saml2.saml import NAME_FORMAT_URI
from saml2.virtual_org import VirtualOrg


logger = logging.getLogger(__name__)

__author__ = 'rolandh'


COMMON_ARGS = [
    "logging",
    "debug",
    "entityid",
    "xmlsec_binary",
    "key_file",
    "cert_file",
    "encryption_keypairs",
    "additional_cert_files",
    "metadata_key_usage",
    "secret",
    "accepted_time_diff",
    "name",
    "ca_certs",
    "description",
    "valid_for",
    "verify_ssl_cert",
    "organization",
    "contact_person",
    "name_form",
    "virtual_organization",
    "only_use_keys_in_metadata",
    "disable_ssl_certificate_validation",
    "preferred_binding",
    "session_storage",
    "assurance_certification",
    "entity_attributes",
    "entity_category",
    "entity_category_support",
    "xmlsec_path",
    "extension_schemas",
    "cert_handler_extra_class",
    "generate_cert_func",
    "generate_cert_info",
    "verify_encrypt_cert_advice",
    "verify_encrypt_cert_assertion",
    "tmp_cert_file",
    "tmp_key_file",
    "validate_certificate",
    "extensions",
    "allow_unknown_attributes",
    "crypto_backend",
    "delete_tmpfiles",
    "endpoints",
    "metadata",
    "ui_info",
    "name_id_format",
    "signing_algorithm",
    "digest_algorithm",
]

SP_ARGS = [
    "required_attributes",
    "optional_attributes",
    "idp",
    "aa",
    "subject_data",
    "want_response_signed",
    "want_assertions_signed",
    "want_assertions_or_response_signed",
    "authn_requests_signed",
    "name_form",
    "discovery_response",
    "allow_unsolicited",
    "ecp",
    "name_id_policy_format",
    "name_id_format_allow_create",
    "logout_requests_signed",
    "logout_responses_signed",
    "requested_attribute_name_format",
    "hide_assertion_consumer_service",
    "force_authn",
    "sp_type",
    "sp_type_in_metadata",
    "requested_attributes",
]

AA_IDP_ARGS = [
    "sign_assertion",
    "sign_response",
    "encrypt_assertion",
    "encrypted_advice_attributes",
    "encrypt_assertion_self_contained",
    "want_authn_requests_signed",
    "want_authn_requests_only_with_valid_cert",
    "provided_attributes",
    "subject_data",
    "sp",
    "scope",
    "domain",
    "name_qualifier",
    "edu_person_targeted_id",
]

PDP_ARGS = ["endpoints", "name_form", "name_id_format"]

AQ_ARGS = ["endpoints"]

AA_ARGS = ["attribute", "attribute_profile"]

COMPLEX_ARGS = ["attribute_converters", "metadata", "policy"]
ALL = set(COMMON_ARGS + SP_ARGS + AA_IDP_ARGS + PDP_ARGS + COMPLEX_ARGS + AA_ARGS)

SPEC = {
    "":    COMMON_ARGS + COMPLEX_ARGS,
    "sp":  COMMON_ARGS + COMPLEX_ARGS + SP_ARGS,
    "idp": COMMON_ARGS + COMPLEX_ARGS + AA_IDP_ARGS,
    "aa":  COMMON_ARGS + COMPLEX_ARGS + AA_IDP_ARGS + AA_ARGS,
    "pdp": COMMON_ARGS + COMPLEX_ARGS + PDP_ARGS,
    "aq":  COMMON_ARGS + COMPLEX_ARGS + AQ_ARGS,
}

_RPA = [BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]
_PRA = [BINDING_HTTP_POST, BINDING_HTTP_REDIRECT, BINDING_HTTP_ARTIFACT]
_SRPA = [BINDING_SOAP, BINDING_HTTP_REDIRECT, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT]

PREFERRED_BINDING = {
    "single_logout_service": _SRPA,
    "manage_name_id_service": _SRPA,
    "assertion_consumer_service": _PRA,
    "single_sign_on_service": _RPA,
    "name_id_mapping_service": [BINDING_SOAP],
    "authn_query_service": [BINDING_SOAP],
    "attribute_service": [BINDING_SOAP],
    "authz_service": [BINDING_SOAP],
    "assertion_id_request_service": [BINDING_URI],
    "artifact_resolution_service": [BINDING_SOAP],
    "attribute_consuming_service": _RPA
}


class ConfigurationError(SAMLError):
    pass


class Config(object):
    def_context = ""

    def __init__(self, homedir="."):
        self.logging = None
        self._homedir = homedir
        self.entityid = None
        self.xmlsec_binary = None
        self.xmlsec_path = []
        self.debug = False
        self.key_file = None
        self.cert_file = None
        self.encryption_keypairs = None
        self.additional_cert_files = None
        self.metadata_key_usage = 'both'
        self.secret = None
        self.accepted_time_diff = None
        self.name = None
        self.ca_certs = None
        self.verify_ssl_cert = False
        self.description = None
        self.valid_for = None
        self.organization = None
        self.contact_person = None
        self.name_form = None
        self.name_id_format = None
        self.name_id_policy_format = None
        self.name_id_format_allow_create = None
        self.virtual_organization = None
        self.only_use_keys_in_metadata = True
        self.logout_requests_signed = None
        self.logout_responses_signed = None
        self.disable_ssl_certificate_validation = None
        self.context = ""
        self.attribute_converters = None
        self.metadata = None
        self.policy = None
        self.serves = []
        self.vorg = {}
        self.preferred_binding = PREFERRED_BINDING
        self.domain = ""
        self.name_qualifier = ""
        self.assurance_certification = []
        self.entity_attributes = []
        self.entity_category = []
        self.entity_category_support = []
        self.crypto_backend = 'xmlsec1'
        self.scope = ""
        self.allow_unknown_attributes = False
        self.extension_schema = {}
        self.cert_handler_extra_class = None
        self.verify_encrypt_cert_advice = None
        self.verify_encrypt_cert_assertion = None
        self.generate_cert_func = None
        self.generate_cert_info = None
        self.tmp_cert_file = None
        self.tmp_key_file = None
        self.validate_certificate = None
        self.extensions = {}
        self.attribute = []
        self.attribute_profile = []
        self.requested_attribute_name_format = NAME_FORMAT_URI
        self.delete_tmpfiles = True
        self.signing_algorithm = None
        self.digest_algorithm = None

    def setattr(self, context, attr, val):
        if context == "":
            setattr(self, attr, val)
        else:
            setattr(self, "_%s_%s" % (context, attr), val)

    def getattr(self, attr, context=None):
        if context is None:
            context = self.context

        if context == "":
            return getattr(self, attr, None)
        else:
            return getattr(self, "_%s_%s" % (context, attr), None)

    def load_special(self, cnf, typ):
        for arg in SPEC[typ]:
            try:
                _val = cnf[arg]
            except KeyError:
                pass
            else:
                if _val == "true":
                    _val = True
                elif _val == "false":
                    _val = False
                self.setattr(typ, arg, _val)

        self.context = typ
        self.context = self.def_context

    def load_complex(self, cnf):
        acs = ac_factory(cnf.get("attribute_map_dir"))
        if not acs:
            raise ConfigurationError("No attribute converters, something is wrong!!")
        self.setattr("", "attribute_converters", acs)

        try:
            self.setattr("", "metadata", self.load_metadata(cnf["metadata"]))
        except KeyError:
            pass

        for srv, spec in cnf.get("service", {}).items():
            policy_conf = spec.get("policy")
            self.setattr(srv, "policy", Policy(policy_conf, self.metadata))

    def load(self, cnf, metadata_construction=None):
        """ The base load method, loads the configuration

        :param cnf: The configuration as a dictionary
        :return: The Configuration instance
        """

        if metadata_construction is not None:
            warn_msg = (
                "The metadata_construction parameter for saml2.config.Config.load "
                "is deprecated and ignored; "
                "instead, initialize the Policy object setting the mds param."
            )
            logger.warning(warn_msg)
            _warn(warn_msg, DeprecationWarning)

        for arg in COMMON_ARGS:
            if arg == "virtual_organization":
                if "virtual_organization" in cnf:
                    for key, val in cnf["virtual_organization"].items():
                        self.vorg[key] = VirtualOrg(None, key, val)
                continue
            elif arg == "extension_schemas":
                # List of filename of modules representing the schemas
                if "extension_schemas" in cnf:
                    for mod_file in cnf["extension_schemas"]:
                        _mod = self._load(mod_file)
                        self.extension_schema[_mod.NAMESPACE] = _mod

            try:
                setattr(self, arg, cnf[arg])
            except KeyError:
                pass
            except TypeError:  # Something that can't be a string
                setattr(self, arg, cnf[arg])

        if self.logging is not None:
            configure_logging_by_dict(self.logging)

        if not self.delete_tmpfiles:
            warn_msg = (
                "Configuration option `delete_tmpfiles` is set to False; "
                "consider setting this to True to have temporary files deleted."
            )
            logger.warning(warn_msg)
            _warn(warn_msg)

        if "service" in cnf:
            for typ in ["aa", "idp", "sp", "pdp", "aq"]:
                try:
                    self.load_special(cnf["service"][typ], typ)
                    self.serves.append(typ)
                except KeyError:
                    pass

        if "extensions" in cnf:
            self.do_extensions(cnf["extensions"])

        self.load_complex(cnf)
        self.context = self.def_context

        return self

    def _load(self, fil):
        head, tail = os.path.split(fil)
        if head == "":
            if sys.path[0] != ".":
                sys.path.insert(0, ".")
        else:
            sys.path.insert(0, head)

        return importlib.import_module(tail)

    def load_file(self, config_filename, metadata_construction=None):
        if metadata_construction is not None:
            warn_msg = (
                "The metadata_construction parameter for saml2.config.Config.load_file "
                "is deprecated and ignored; "
                "instead, initialize the Policy object setting the mds param."
            )
            logger.warning(warn_msg)
            _warn(warn_msg, DeprecationWarning)

        if config_filename.endswith(".py"):
            config_filename = config_filename[:-3]

        mod = self._load(config_filename)
        return self.load(copy.deepcopy(mod.CONFIG))

    def load_metadata(self, metadata_conf):
        """ Loads metadata into an internal structure """

        acs = self.attribute_converters
        if acs is None:
            raise ConfigurationError("Missing attribute converter specification")

        try:
            ca_certs = self.ca_certs
        except:
            ca_certs = None
        try:
            disable_validation = self.disable_ssl_certificate_validation
        except:
            disable_validation = False

        mds = MetadataStore(acs, self, ca_certs,
            disable_ssl_certificate_validation=disable_validation)

        mds.imp(metadata_conf)

        return mds

    def endpoint(self, service, binding=None, context=None):
        """ Goes through the list of endpoint specifications for the
        given type of service and returns a list of endpoint that matches
        the given binding. If no binding is given all endpoints available for
        that service will be returned.

        :param service: The service the endpoint should support
        :param binding: The expected binding
        :return: All the endpoints that matches the given restrictions
        """
        spec = []
        unspec = []
        endps = self.getattr("endpoints", context)
        if endps and service in endps:
            for endpspec in endps[service]:
                try:
                    endp, bind = endpspec
                    if binding is None or bind == binding:
                        spec.append(endp)
                except ValueError:
                    unspec.append(endpspec)

        if spec:
            return spec
        else:
            return unspec

    def endpoint2service(self, endpoint, context=None):
        endps = self.getattr("endpoints", context)

        for service, specs in endps.items():
            for endp, binding in specs:
                if endp == endpoint:
                    return service, binding

        return None, None

    def do_extensions(self, extensions):
        for key, val in extensions.items():
            self.extensions[key] = val

    def service_per_endpoint(self, context=None):
        """
        List all endpoint this entity publishes and which service and binding
        that are behind the endpoint

        :param context: Type of entity
        :return: Dictionary with endpoint url as key and a tuple of
            service and binding as value
        """
        endps = self.getattr("endpoints", context)
        res = {}
        for service, specs in endps.items():
            for endp, binding in specs:
                res[endp] = (service, binding)
        return res


class SPConfig(Config):
    def_context = "sp"

    def __init__(self):
        Config.__init__(self)

    def vo_conf(self, vo_name):
        try:
            return self.virtual_organization[vo_name]
        except KeyError:
            return None

    def ecp_endpoint(self, ipaddress):
        """
        Returns the entity ID of the IdP which the ECP client should talk to

        :param ipaddress: The IP address of the user client
        :return: IdP entity ID or None
        """
        _ecp = self.getattr("ecp")
        if _ecp:
            for key, eid in _ecp.items():
                if re.match(key, ipaddress):
                    return eid

        return None


class IdPConfig(Config):
    def_context = "idp"

    def __init__(self):
        Config.__init__(self)


def config_factory(_type, config):
    """

    :type _type: str
    :param _type:

    :type config: str or dict
    :param config: Name of file with pysaml2 config or CONFIG dict

    :return:
    """
    if _type == "sp":
        conf = SPConfig()
    elif _type in ["aa", "idp", "pdp", "aq"]:
        conf = IdPConfig()
    else:
        conf = Config()

    if isinstance(config, dict):
        conf.load(copy.deepcopy(config))
    elif isinstance(config, str):
        conf.load_file(config)
    else:
        raise ValueError('Unknown type of config')

    conf.context = _type
    return conf



#!/usr/bin/env python

#
# Generated Mon May  2 14:23:33 2011 by parse_xsd.py version 0.4.
#

import saml2
from saml2 import SamlBase

from saml2 import md

NAMESPACE = 'urn:oasis:names:tc:SAML:metadata:ui'


class DisplayName(md.LocalizedNameType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:DisplayName element """

    c_tag = 'DisplayName'
    c_namespace = NAMESPACE
    c_children = md.LocalizedNameType_.c_children.copy()
    c_attributes = md.LocalizedNameType_.c_attributes.copy()
    c_child_order = md.LocalizedNameType_.c_child_order[:]
    c_cardinality = md.LocalizedNameType_.c_cardinality.copy()


def display_name_from_string(xml_string):
    return saml2.create_class_from_xml_string(DisplayName, xml_string)


class Description(md.LocalizedNameType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:Description element """

    c_tag = 'Description'
    c_namespace = NAMESPACE
    c_children = md.LocalizedNameType_.c_children.copy()
    c_attributes = md.LocalizedNameType_.c_attributes.copy()
    c_child_order = md.LocalizedNameType_.c_child_order[:]
    c_cardinality = md.LocalizedNameType_.c_cardinality.copy()


def description_from_string(xml_string):
    return saml2.create_class_from_xml_string(Description, xml_string)


class InformationURL(md.LocalizedURIType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:InformationURL element """

    c_tag = 'InformationURL'
    c_namespace = NAMESPACE
    c_children = md.LocalizedURIType_.c_children.copy()
    c_attributes = md.LocalizedURIType_.c_attributes.copy()
    c_child_order = md.LocalizedURIType_.c_child_order[:]
    c_cardinality = md.LocalizedURIType_.c_cardinality.copy()


def information_url_from_string(xml_string):
    return saml2.create_class_from_xml_string(InformationURL, xml_string)


class PrivacyStatementURL(md.LocalizedURIType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:PrivacyStatementURL element """

    c_tag = 'PrivacyStatementURL'
    c_namespace = NAMESPACE
    c_children = md.LocalizedURIType_.c_children.copy()
    c_attributes = md.LocalizedURIType_.c_attributes.copy()
    c_child_order = md.LocalizedURIType_.c_child_order[:]
    c_cardinality = md.LocalizedURIType_.c_cardinality.copy()


def privacy_statement_url_from_string(xml_string):
    return saml2.create_class_from_xml_string(PrivacyStatementURL, xml_string)


class ListOfStrings_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:listOfStrings element """

    c_tag = 'listOfStrings'
    c_namespace = NAMESPACE
    c_value_type = {'member': 'string', 'base': 'list'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def list_of_strings__from_string(xml_string):
    return saml2.create_class_from_xml_string(ListOfStrings_, xml_string)


class KeywordsType_(ListOfStrings_):
    """The urn:oasis:names:tc:SAML:metadata:ui:KeywordsType element """

    c_tag = 'KeywordsType'
    c_namespace = NAMESPACE
    c_children = ListOfStrings_.c_children.copy()
    c_attributes = ListOfStrings_.c_attributes.copy()
    c_child_order = ListOfStrings_.c_child_order[:]
    c_cardinality = ListOfStrings_.c_cardinality.copy()
    c_attributes['{http://www.w3.org/XML/1998/namespace}lang'] = (
    'lang', 'mdui:listOfStrings', True)

    def __init__(self,
                 lang=None,
                 text=None,
                 extension_elements=None,
                 extension_attributes=None):
        ListOfStrings_.__init__(self,
                                text=text,
                                extension_elements=extension_elements,
                                extension_attributes=extension_attributes)
        self.lang = lang


def keywords_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(KeywordsType_, xml_string)


class LogoType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:LogoType element """

    c_tag = 'LogoType'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes['height'] = ('height', 'positiveInteger', True)
    c_attributes['width'] = ('width', 'positiveInteger', True)
    c_attributes['{http://www.w3.org/XML/1998/namespace}lang'] = (
        'lang', 'anyURI', False)

    def __init__(self,
                 height=None,
                 width=None,
                 lang=None,
                 text=None,
                 extension_elements=None,
                 extension_attributes=None):
        SamlBase.__init__(self,
                          text=text,
                          extension_elements=extension_elements,
                          extension_attributes=extension_attributes)
        self.height = height
        self.width = width
        self.lang = lang


def logo_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(LogoType_, xml_string)


class IPHint(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:IPHint element """

    c_tag = 'IPHint'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def ip_hint_from_string(xml_string):
    return saml2.create_class_from_xml_string(IPHint, xml_string)


class DomainHint(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:DomainHint element """

    c_tag = 'DomainHint'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'string'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def domain_hint_from_string(xml_string):
    return saml2.create_class_from_xml_string(DomainHint, xml_string)


class GeolocationHint(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:GeolocationHint element """

    c_tag = 'GeolocationHint'
    c_namespace = NAMESPACE
    c_value_type = {'base': 'anyURI'}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def geolocation_hint_from_string(xml_string):
    return saml2.create_class_from_xml_string(GeolocationHint, xml_string)


class Keywords(KeywordsType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:Keywords element """

    c_tag = 'Keywords'
    c_namespace = NAMESPACE
    c_children = KeywordsType_.c_children.copy()
    c_attributes = KeywordsType_.c_attributes.copy()
    c_child_order = KeywordsType_.c_child_order[:]
    c_cardinality = KeywordsType_.c_cardinality.copy()


def keywords_from_string(xml_string):
    return saml2.create_class_from_xml_string(Keywords, xml_string)


class Logo(LogoType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:Logo element """

    c_tag = 'Logo'
    c_namespace = NAMESPACE
    c_children = LogoType_.c_children.copy()
    c_attributes = LogoType_.c_attributes.copy()
    c_child_order = LogoType_.c_child_order[:]
    c_cardinality = LogoType_.c_cardinality.copy()


def logo_from_string(xml_string):
    return saml2.create_class_from_xml_string(Logo, xml_string)


class DiscoHintsType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:DiscoHintsType element """

    c_tag = 'DiscoHintsType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}IPHint'] = (
        'ip_hint', [IPHint])
    c_cardinality['ip_hint'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}DomainHint'] = (
        'domain_hint', [DomainHint])
    c_cardinality['domain_hint'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}GeolocationHint'] = (
        'geolocation_hint', [GeolocationHint])
    c_cardinality['geolocation_hint'] = {"min": 0}
    c_child_order.extend(['ip_hint', 'domain_hint', 'geolocation_hint'])

    def __init__(self,
                 ip_hint=None,
                 domain_hint=None,
                 geolocation_hint=None,
                 text=None,
                 extension_elements=None,
                 extension_attributes=None):
        SamlBase.__init__(self,
                          text=text,
                          extension_elements=extension_elements,
                          extension_attributes=extension_attributes)
        self.ip_hint = ip_hint or []
        self.domain_hint = domain_hint or []
        self.geolocation_hint = geolocation_hint or []


def disco_hints_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(DiscoHintsType_, xml_string)


class UIInfoType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:ui:UIInfoType element """

    c_tag = 'UIInfoType'
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}DisplayName'] = (
        'display_name', [DisplayName])
    c_cardinality['display_name'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}Description'] = (
        'description', [Description])
    c_cardinality['description'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}Keywords'] = (
        'keywords', [Keywords])
    c_cardinality['keywords'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}Logo'] = ('logo', [Logo])
    c_cardinality['logo'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}InformationURL'] = (
        'information_url', [InformationURL])
    c_cardinality['information_url'] = {"min": 0}
    c_children['{urn:oasis:names:tc:SAML:metadata:ui}PrivacyStatementURL'] = (
        'privacy_statement_url', [PrivacyStatementURL])
    c_cardinality['privacy_statement_url'] = {"min": 0}
    c_child_order.extend(
        ['display_name', 'description', 'keywords', 'logo', 'information_url',
         'privacy_statement_url'])

    def __init__(self,
                 display_name=None,
                 description=None,
                 keywords=None,
                 logo=None,
                 information_url=None,
                 privacy_statement_url=None,
                 text=None,
                 extension_elements=None,
                 extension_attributes=None):
        SamlBase.__init__(self,
                          text=text,
                          extension_elements=extension_elements,
                          extension_attributes=extension_attributes)
        self.display_name = display_name or []
        self.description = description or []
        self.keywords = keywords or []
        self.logo = logo or []
        self.information_url = information_url or []
        self.privacy_statement_url = privacy_statement_url or []


def ui_info_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(UIInfoType_, xml_string)


class DiscoHints(DiscoHintsType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:DiscoHints element """

    c_tag = 'DiscoHints'
    c_namespace = NAMESPACE
    c_children = DiscoHintsType_.c_children.copy()
    c_attributes = DiscoHintsType_.c_attributes.copy()
    c_child_order = DiscoHintsType_.c_child_order[:]
    c_cardinality = DiscoHintsType_.c_cardinality.copy()


def disco_hints_from_string(xml_string):
    return saml2.create_class_from_xml_string(DiscoHints, xml_string)


class UIInfo(UIInfoType_):
    """The urn:oasis:names:tc:SAML:metadata:ui:UIInfo element """

    c_tag = 'UIInfo'
    c_namespace = NAMESPACE
    c_children = UIInfoType_.c_children.copy()
    c_attributes = UIInfoType_.c_attributes.copy()
    c_child_order = UIInfoType_.c_child_order[:]
    c_cardinality = UIInfoType_.c_cardinality.copy()


def ui_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(UIInfo, xml_string)


ELEMENT_FROM_STRING = {
    UIInfo.c_tag: ui_info_from_string,
    UIInfoType_.c_tag: ui_info_type__from_string,
    DisplayName.c_tag: display_name_from_string,
    Description.c_tag: description_from_string,
    InformationURL.c_tag: information_url_from_string,
    PrivacyStatementURL.c_tag: privacy_statement_url_from_string,
    Keywords.c_tag: keywords_from_string,
    KeywordsType_.c_tag: keywords_type__from_string,
    ListOfStrings_.c_tag: list_of_strings__from_string,
    Logo.c_tag: logo_from_string,
    LogoType_.c_tag: logo_type__from_string,
    DiscoHints.c_tag: disco_hints_from_string,
    DiscoHintsType_.c_tag: disco_hints_type__from_string,
    IPHint.c_tag: ip_hint_from_string,
    DomainHint.c_tag: domain_hint_from_string,
    GeolocationHint.c_tag: geolocation_hint_from_string,
}

ELEMENT_BY_TAG = {
    'UIInfo': UIInfo,
    'UIInfoType': UIInfoType_,
    'DisplayName': DisplayName,
    'Description': Description,
    'InformationURL': InformationURL,
    'PrivacyStatementURL': PrivacyStatementURL,
    'Keywords': Keywords,
    'KeywordsType': KeywordsType_,
    'listOfStrings': ListOfStrings_,
    'Logo': Logo,
    'LogoType': LogoType_,
    'DiscoHints': DiscoHints,
    'DiscoHintsType': DiscoHintsType_,
    'IPHint': IPHint,
    'DomainHint': DomainHint,
    'GeolocationHint': GeolocationHint,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



from six.moves.urllib import parse

from saml2.entity import Entity
from saml2.response import VerificationError

__author__ = 'rolandh'

IDPDISC_POLICY = "urn:oasis:names:tc:SAML:profiles:SSO:idp-discovery-protocol:single"


class DiscoveryServer(Entity):
    def __init__(self, config=None, config_file=""):
        if config or config_file:
            Entity.__init__(self, "disco", config, config_file)

    def parse_discovery_service_request(self, url="", query=""):
        if url:
            part = parse.urlparse(url)
            dsr = parse.parse_qs(part[4])
        elif query:
            dsr = parse.parse_qs(query)
        else:
            dsr = {}

        # verify

        for key in ["isPassive", "return", "returnIDParam", "policy", 'entityID']:
            try:
                if len(dsr[key]) != 1:
                    raise Exception("Invalid DS request keys: {k}".format(k=key))
                dsr[key] = dsr[key][0]
            except KeyError:
                pass

        if "return" in dsr:
            part = parse.urlparse(dsr["return"])
            if part.query:
                qp = parse.parse_qs(part.query)
                if "returnIDParam" in dsr:
                    if dsr["returnIDParam"] in qp.keys():
                        raise Exception(
                            "returnIDParam value should not be in the query params"
                        )
                else:
                    if "entityID" in qp.keys():
                        raise Exception("entityID should not be in the query params")
        else:
            # If metadata not used this is mandatory
            raise VerificationError("Missing mandatory parameter 'return'")

        if "policy" not in dsr:
            dsr["policy"] = IDPDISC_POLICY

        is_passive = dsr.get("isPassive")
        if is_passive not in ["true", "false"]:
            raise ValueError(
                "Invalid value '{v}' for attribute '{attr}'".format(
                    v=is_passive, attr="isPassive"
                )
            )

        if "isPassive" in dsr and dsr["isPassive"] == "true":
            dsr["isPassive"] = True
        else:
            dsr["isPassive"] = False

        if not "returnIDParam" in dsr:
            dsr["returnIDParam"] = "entityID"

        return dsr

    # -------------------------------------------------------------------------

    @staticmethod
    def create_discovery_service_response(return_url=None,
                                          returnIDParam="entityID",
                                          entity_id=None, **kwargs):
        if return_url is None:
            return_url = kwargs["return"]

        if entity_id:
            qp = parse.urlencode({returnIDParam: entity_id})

            part = parse.urlparse(return_url)
            if part.query:
                # Iff there is a query part add the new info at the end
                return_url = "%s&%s" % (return_url, qp)
            else:
                return_url = "%s?%s" % (return_url, qp)

        return return_url

    def verify_sp_in_metadata(self, entity_id):
        if self.metadata:
            endp = self.metadata.discovery_response(entity_id)
            if endp:
                return True

        return False

    def verify_return(self, entity_id, return_url):
        for endp in self.metadata.discovery_response(entity_id):
            if not return_url.startswith(endp["location"]):
                return True
        return False
